// ボタンの初期化
function initBtn() {
	new InfoBtn(document.getElementById("infoBtn"),"white");
	createGenericButton(document.getElementById("doneBtn"), getLocalizedString("Done"), function () {
		// 裏→表返し
		if (window.widget) widget.prepareForTransition("ToFront");
		document.getElementById("back").style.display="none";
		document.getElementById("front").style.display="block";
		if (window.widget) setTimeout('widget.performTransition();if (onFlipFront) onFlipFront();', 0);
	});
}

//-- Prefオブジェクト --
// 初期設定関連のメソッドをまとめたユーティリティーオブジェクト
// オブジェクトを生成せず、Mathみたい使う
// value属性を持ち、初期設定keyと同じidの要素が必要
// valueを持たない要素は、別途処理すること
var Pref = {
	// 初期設定を読み込む
	// aKey:初期値のKey(id)、aDef:デフォルト値
	load : function (aKey, aDef) {
		if (window.widget) {
			var v = widget.preferenceForKey(aKey);
			if (v == undefined) v = aDef;
		} else {
			var v = aDef;
		}
		var theObj = document.getElementById(aKey);
		switch (theObj.type) {
		case "checkbox" :
			theObj.checked = v;
			break;
		default :
			theObj.value = v;
		}
	},
	
	// 初期設定の書き出し
	save : function (aKey) {
		if (!window.widget) return;
		var theObj = document.getElementById(aKey);
		switch (theObj.type) {
		case "checkbox" :
			widget.setPreferenceForKey(theObj.checked,aKey);
			break;
		default :
			widget.setPreferenceForKey(theObj.value,aKey);
		}
	},
	
	// 初期設定のチェック
	getValue : function (aKey) {
		var theObj = document.getElementById(aKey);
		switch (theObj.type) {
		case "checkbox" :
			return theObj.checked;
		default :
			return theObj.value;
		}
	}
}

//-- InfoBtnオブジェクト --
// オブジェクトのコンストラクタなので、newで呼ぶこと
// addEventListenerとsetIntervalに指定する関数の持つスコープが分かりにくく
// なんだかよく分からんオブジェクト構造になっている
function InfoBtn(aElement,aColor) {
	if (aColor == undefined) aColor = "white";
	this.init(aElement, aColor);
}

InfoBtn.prototype = {
	duration : 500, starttime : 0, now : 0.0, from : 0.0, to : 1.0, timer : null,
	init : function (aElement, aColor) {
		var baseStyle = "display:inline-block;background-repeat:no-repeat;width:13px;height:13px;position:absolute;top:0px;left:0px;";
		var filePath = "file:///System/Library/WidgetResources/ibutton/"+aColor;

		// iアイコン
		this.flip = document.createElement('span');
		this.flip.setAttribute("style", baseStyle + 'background:url(' + filePath + '_i.png);z-index:8000;opacity:0;');
		aElement.appendChild(this.flip);

		// 丸アイコン
		var fliprollie = document.createElement('span');
		fliprollie.setAttribute("style", baseStyle + 'background:url(' + filePath + '_rollie.png);display:none;opacity:0.25;z-index:7999;');
		aElement.appendChild(fliprollie);

		// ( i )ボタンにイベントを設定
		aElement.addEventListener("mouseover", function () {fliprollie.style.display = 'block';}, true);
		aElement.addEventListener("mouseout", function () {fliprollie.style.display = 'none';}, true);
		aElement.addEventListener("mouseup", function () {
			// 表→裏返し
			if (onFlipBack) onFlipBack();
			if (window.widget) widget.prepareForTransition("ToBack");
			document.getElementById("front").style.display="none";
			document.getElementById("back").style.display="block";
			if (window.widget) setTimeout('widget.performTransition();', 0);
			fliprollie.style.display = 'none';
			//event.stopPropagation();
			//event.preventDefault();
		}, true);

		// 表パネルにイベントを設定
		var thisInfoBtn = this; // イベントで呼び出された場合、thisが呼び出し元要素になるので、別に用意する
		var frontPanel = document.getElementById("front");
		frontPanel.addEventListener("mouseover", function () {thisInfoBtn.initAnimate(1.0);}, true);
		frontPanel.addEventListener("mouseout", function () {thisInfoBtn.initAnimate(0.0);}, true);
	},

	initAnimate : function (finalOpacity) {
	// アニメ前初期化
		with (this) {
			if (timer != null) clearInterval(timer);
			starttime = new Date().getTime();// 開始時間
			// 一定時間ごとのアニメ関数呼び出し。
			// animateを、直接置いたんじゃ動かないfunctionでくるむと動く。
			timer = setInterval(function () {animate();}, 13);
			from = 	now;	// beginning opacity (not ness. 0)
			to = finalOpacity;			// final opacity
		}
	},

	animate : function () {
	// フェードアニメを実行
		with (this) {
			var t = new Date().getTime() - starttime;
			if (t >= duration) {
				clearInterval(timer);
				timer = null;
				now = to;
			} else {
				now = from + (to-from)*(0.5 - (0.5 * Math.cos(Math.PI * t / duration)));
			}
			flip.style.opacity = now;
		}
	}
}

//-- ローカライズ関連 --

// ローカライズされた文字列を得る
function getLocalizedString(aKey) {
    try {
		var v = localizedStrings[aKey];
		if ( v != undefined ) return v;
    } catch (ex) {}
    return aKey;
}

// ラベルの設定
// 要素のIDとローカライズ文字列のkeyが同じである必要がある
function setLabel(aKey) {
	var obj = document.getElementById(aKey);
	obj.innerText = getLocalizedString(aKey);
}

// デバッグ用出力
function debug(str) {
	document.getElementById("debug").innerText = str;
}